﻿-------------------------------------------------------------------------------
-- SilentSendAddonMessage.lua
--
-- A simple hook into the SendAddonMessage API
-- to keep it from generating error messages for bad channels
-- such as "you are not in a guild" and "no such player as 'player' is online"
--
-- This is intended to hide the issue of interrupted data transfers
-- when sending a queue of data in whispers to a player who goes offline.
-- or when leaving guild during a similar bulk data transfer
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- INIT
-------------------------------------------------------------------------------

local _G = _G
local VERSION = 2 -- APR 27, 2010

-- only load once
if _G.SilentSendAddonMessage then
	-- try to unhook an older version, if applicable
	if _G.SilentSendAddonMessage.version < VERSION then
		_G.SendAddonMessage = SilentSendAddonMessage.Orig_SendAddonMessage
		_G.SilentSendAddonMessage = nil --fully unload
	else
		-- an up to date version is already loaded
		return
	end
end

-- create
_G.SilentSendAddonMessage = {}
_G.SilentSendAddonMessage.version = VERSION -- for future use


-------------------------------------------------------------------------------
-- THE SILENCER FUNCTION
-------------------------------------------------------------------------------


-- i'm not a fan of if-elseif if i could do a switch or table of functions instead
-- but this code is optimized for performance (I hope!) because it will be called frequently
_G.SilentSendAddonMessage.func = function ( prefix, text, chattype, target )
	if chattype == "WHISPER" and not UnitIsConnected( target) then
		-- attempt to whisper to a player who doesn't exist
		----print("SilentSendAddonMessage: player is not online:"..tostring(target))
		return
	elseif chattype == "GUILD" and not IsInGuild() then
		-- attempt to send to GUILD when you're not in a guild
		----print("SilentSendAddonMessage: you're not in a guild")
		return
	elseif chattype == "RAID" then
		if not GetNumRaidMembers() then --nil or 0
			--NOTE: GetNumRealRaidMembers() appeared to be blocking PallyPower in WoW 3.3.3
			----print("SilentSendAddonMessage: RAID GetNumRaidMembers prefix:"..tostring(prefix))
			--nobody else in the raid, or we just left the raid
			--return
		elseif (GetNumBattlefieldScores() > 0) then --BG in progress, RAID channel is not valid
			----print("SilentSendAddonMessage: RAID GetNumBattlefieldScores prefix:"..tostring(prefix))
			--return
		elseif IsActiveBattlefieldArena() then --Arena in progress, RAID channel is not valid
			----print("SilentSendAddonMessage: RAID IsActiveBattlefieldArena prefix:"..tostring(prefix))
			--return
		else
			--NOTE: GetNumBattlefieldScores returns 0 during the prep time... 
			--	 not anymore - fixed by Blizz in 3.3.3
			--[[ mothballed
			-- in order to recognize that we're in the BG prep time and still treat it as a BG
			-- we have to check all possible queues to see if our status is now "active" in one of them
			local status, mapName, instanceID, minlevel, maxlevel;
			for i=1, MAX_BATTLEFIELD_QUEUES do
				--status, mapName, instanceID, minlevel, maxlevel, teamSize = GetBattlefieldStatus(i);
				local status = GetBattlefieldStatus(i)
				if status == "active" then
					-- active in a BG prep time, RAID channel is not valid
					print("SilentSendAddonMessage: RAID GetBattlefieldStatus prefix:"..tostring(prefix))
					return
				end
			end

			-- else, WoW 3.3.3 doesn't complain about sending to RAID unless you're in a BG
			--		even if you're not in a RAID group... WTH?!
			--		so we'll allow it, fall through...
			
			end mothball --]]
		end
	end
	-- else, we didn't abort above
	
	-- the chattype and target look valid
	--print("SilentSendAddonMessage: allowing prefix:"..tostring(prefix))
	return SilentSendAddonMessage.Orig_SendAddonMessage( prefix, text, chattype, target )
end


-------------------------------------------------------------------------------
-- REPLACE global SendAddonMessage with our silent version
-------------------------------------------------------------------------------

-- hooksecurefunc would call our function AFTER the original function
-- but we need to be called first, so we can abort the original call

_G.SilentSendAddonMessage.Orig_SendAddonMessage = _G.SendAddonMessage
_G.SendAddonMessage = SilentSendAddonMessage.func



-------------------------------------------------------------------------------
-- FOR FUTURE USE
-------------------------------------------------------------------------------

-- sample code and notes 
-- for reference purposes in case blizzard changes it on us

-- Currently (WoW 3.2.2) the party, raid, and battleground channels 
-- do not generate error messages from the blizzard API, 
-- even if they are invalid channels, i.e. because you left the group
-- so we don't bother to validate those channels in the simpler function above

-- the table-driven function below seemed slightly slower 
-- and it's unnecessary since the group channels don't show error messages

--[[


local SAFETYCHECKS = {

["WHISPER"]		= function( t ) 
	--NOTE: no handling here for error from attempting cross-faction, 
	-- but that's not something that will change in the middle of processing a queue
	return UnitIsConnected( t ) 
end,

["GUILD"]		= function( t ) 
	return IsInGuild() 
end,

["PARTY"]		= function( t ) 
	-- GetNumRealPartyMembers() > 1
	-- WoW 3.2.2 is silent on this error, so we can just allow it
	return true 
end,

["RAID"]		= function( t ) 
	-- GetNumRealRaidMembers() > 1
	-- WoW 3.2.2 is silent on this error, so we can just allow it
	--	3.3.3 is not, apparently related to battleground groups which are not "raids"
	return true 
end,

["BATTLEGROUND"]= function( t ) 
	-- this check would require a loop to correctly detect prep time
	-- WoW 3.2.2 is silent on this error, so we can just allow it
	return true
end,

} -- end SAFETYCHECKS


_G.SilentSendAddonMessage.func = function ( prefix, text, chattype, target )
    if not SAFETYCHECKS[ chattype ]( target ) then
        print("SilentSendAddonMessage: bad target: "..tostring(target or chattype) )
        return
    end
    return SilentSendAddonMessage.Orig_SendAddonMessage( prefix, text, chattype, target )
end


--]]
